# Application imports
from apitax.drivers.DriverCommands import DriverCommands


# from apitax.drivers.plugins.commandtax.apitaxtests import *

# Openstack Command Driver for handling custom commands when the openstack driver is used
class ApitaxInfoCommands(DriverCommands):

    def getDriverName(self):
        return 'apitaxinfo'



from apitax.integrations.BasicGit import BasicGit as Git
from apitax.drivers.Driver import Driver
from apitax.utilities.Files import getAllFiles
from apitax.utilities.Files import getPath
from apitax.utilities.Files import createDir
from pathlib import Path


class BasicGitDriver(Driver):
    def isApiAuthenticated(self):
        return False

    def isTokenable(self):
        return False

    def getScriptsCatalog(self):
        path = self.getScriptsPath()
        createDir(path)
        git = Git()
        git.clone(self.driverConfig.get('git-repo-clone-url'), path)
        # print(self.driverConfig.get('git-repo-clone-url'))
        files = getAllFiles(path + "/**/*.ah")
        returner = {"scripts": []}
        for file in files:
            returner['scripts'].append({"label": file.split('/')[-1].split('.')[0].title(), "relative-path": file,
                                        "path": str(Path(file).resolve())})
        # print(returner)
        return returner



from apitax.drivers.Driver import Driver
from apitax.utilities.Files import getAllFiles
from pathlib import Path


class ApitaxTestsDriver(Driver):
    def isApiAuthenticated(self):
        return False

    def isTokenable(self):
        return False

    def getScriptsCatalog(self):
        files = getAllFiles(self.config.path + "/grammar/scripts/**/*.ah")
        returner = {"scripts": []}
        for file in files:
            returner['scripts'].append({"label": file.split('/')[-1].split('.')[0].title(), "relative-path": file,
                                        "path": str(Path(file).resolve())})
        # print(returner)
        return returner

    def getCatalog(self, auth):
        catalog = super().getCatalog(auth)
        catalog['endpoints'].update({"tests": {"label": "(Test) JSONPlaceHolder - https://jsonplaceholder.typicode.com",
                                               "value": "https://jsonplaceholder.typicode.com"}})
        catalog['endpoints'].update(
            {"reqres": {"label": "(Test) Requests - https://reqres.in/", "value": "https://reqres.in/"}})

        catalog['selected'] = "https://reqres.in/"

        return catalog



from apitax.drivers.Driver import Driver
from apitax.utilities.Json import read
from apitax.ah.State import State
from apitax.utilities.Files import getPath


class BasicAuthDriver(Driver):

    def __init__(self):
        super().__init__()
        self.users = read(getPath(State.paths['root'] + "/app/users.json"))

    def isApiAuthenticated(self):
        return False

    def isTokenable(self):
        return False



# Application imports
from apitax.drivers.DriverCommands import DriverCommands
from apitax.ah.State import State

# from apitax.drivers.plugins.commandtax.apitaxtests import *

# Openstack Command Driver for handling custom commands when the openstack driver is used
class AnsibleCommands(DriverCommands):

    def getDriverName(self):
        return 'ansible'
        
    def getResponseBody(self):
        State.log.log("yes")
        return 'winner takes all'

    def handle(self, command):
        State.log.log(command)
        return self



from apitax.drivers.Driver import Driver
from apitax.utilities.Files import getAllFiles
from apitax.ah.Options import Options
from pathlib import Path
from apitax.ah.Credentials import Credentials
from apitax.utilities.Json import read
from apitax.ah.State import State
from apitax.utilities.Files import getPath


class VsrxDriver(Driver):

    def __init__(self):
        super().__init__()
        self.users = read(getPath(State.paths['root'] + "/app/users.json"))

    def isCredentialsPosted(self):
        return False

    def isConfigurable(self):
        return True

    def isTokenable(self):
        return False
        
    def isApiAuthenticated(self):
        return False
        
    def piggyBackOffApiAuth(self):
        return True

    def getContentTypeJSON(self):
        # if(not self.isAuthenticated()):
        #   return {}
        return {'Content-Type': 'application/xml'}

    def getCatalog(self, auth):
        catalog = {'endpoints': {'demo': {'label': 'demo', 'value': 'http://172.25.190.59:3000'}}}
        catalog['selected'] = 'http://172.25.190.59:3000'
        return catalog

    def apitaxAuth(self, authObj):
        authObj = authObj['credentials']
        authRequest = ['apiAuthRequest']
        #print(authRequest)
        try:
            return self.users[authObj.username]['role']
        except:
            return None
        return None


# Application import
from apitax.ah.commandtax.commands.Script import Script
from pathlib import Path


# Handles Openstack commands related to domains
class Tests(Script):
    def __init__(self, config, header, parameters, debug, sensitive):
        Script.__init__(self, config, header, parameters, debug, sensitive)

    def handle(self, command):
        command = str(
            Path('apitax/drivers/plugins/commandtax/apitaxtests/tests/' + '_'.join(command) + '.ah').resolve())
        super().handle([command])



# Application imports
from apitax.drivers.DriverCommands import DriverCommands
from apitax.ah.State import State

# from apitax.drivers.plugins.commandtax.apitaxtests import *

# Openstack Command Driver for handling custom commands when the openstack driver is used
class AnsibleCommands(DriverCommands):

    def getDriverName(self):
        return 'ansible'


    def handle(self, command):
        import subprocess
        #call(["ansible-playbook"] + command)

        proc = subprocess.Popen(["ansible-playbook"] + command[:-2], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = proc.communicate()
        out = str(out.decode("utf-8"))
        err = str(err.decode("utf-8"))
        #print ('stdout:'+ str(out))
        #print ('stderr:'+str(err))
        return Response(status=200, body = "out: " + out + " ### err: " + err)
        #return self




class Response:

    def __init__(self, status=None, body=None):
        self.status = status
        self.body = body
	
    def getResponseBody(self):
        return self.body
        
    def getResponseStatusCode(self):
        return self.status


from scriptax.catalogs.ScriptCatalog import ScriptCatalog

from apitaxdrivers.integrations.Github import Github
from scriptax.drivers.builtin.Scriptax import Scriptax


class GithubDriver(Scriptax):

    def __init__(self):
        super().__init__()
        self.git = Github(self.driverConfig.get('personal-access-token'), self.driverConfig.get('repo'))

    def isDriverConfigurable(self) -> bool:
        return True

    def getDriverName(self) -> str:
        return "github"

    def getDriverDescription(self) -> str:
        return "Provides access to a github repository for utilizing scripts"

    def getDriverHelpEndpoint(self) -> str:
        return "coming soon"

    def getDriverTips(self) -> str:
        return "coming soon"

    def getDriverScriptCatalog(self) -> ScriptCatalog:
        catalog = ScriptCatalog()
        for file in self.git.getFiles():
            path = file.path
            if path[-3:] == '.ah':
                catalog.add(label=path.split('/')[-1].split('.')[0].title(), path=self.git.getPath(path))
        return catalog

    def getDriverScript(self, path) -> str:
        return self.git.getFileContent(path)

    def renameDriverScript(self, original, now) -> bool:
        return self.git.renameFile(original, now)

    def saveDriverScript(self, path, content) -> bool:
        return self.git.createOrUpdate(path, content)

    def deleteDriverScript(self, path) -> bool:
        return self.git.deleteFile(path=path)



from apitax.drivers.Driver import Driver
from apitax.utilities.Files import getAllFiles
from apitax.ah.Options import Options
from pathlib import Path
from apitax.ah.Credentials import Credentials


class AnsibleDriver(Driver):
    def isApiAuthenticated(self):
        return False

    def isTokenable(self):
        return False



from apitax.integrations.Git import Git

import gitlab


class Gitlab(Git):
    def __init__(self, config):
        if (config.has('gitlab-access-token')):
            super().__init__(gitlab.Gitlab('https://142.164.8.254', private_token=config.get('gitlab-access-token')))

    def getRepos(self):
        projects = self.git.projects.list()
        for project in projects:
            print(project)



import git
from git import Repo


class BasicGit:

    def push(self):
        pass

    def pull(self, path):
        repo = git.Repo(path)
        o = repo.remotes.origin
        o.pull()

    def clone(self, url, path):
        if (self.isGitRepo(path)):
            self.pull(path)
        else:
            Repo.clone_from(url, path)

    def isGitRepo(self, path):
        try:
            _ = git.Repo(path).git_dir
            return True
        except git.exc.InvalidGitRepositoryError:
            return False

    def commit(self):
        pass

    def inPush(self):
        pass

    def getRepo(self, name):
        pass

    def getRepos(self):
        pass

    def getUser(self):
        pass



from apitaxdrivers.integrations.Git import Git

from github import Github as GithubIntegration


class Github(Git):
    def __init__(self, token, repo=None):
        super().__init__()
        self.git = GithubIntegration(token)
        self.repo = None
        if (repo):
            self.setRepo(repo)
        self.user = self.git.get_user()

    # This requires a full name: "ShawnClake/Apitax"
    def setRepo(self, name):
        self.repo = self.git.get_repo(name)
        return self.repo

    def getRepos(self):
        return self.git.get_user().get_repos()

    def printRepos(self):
        for repo in self.getRepos():
            print(repo.name)

    def isFileExists(self, path):
        try:
            files = self.getFiles(path, recursive=False)
            if (len(files) > 1):
                return False
            if (len(files) < 1):
                return False
            if (files[0].type == 'dir'):
                return False
            return files[0]
        except:
            return False

    def createFile(self, path, content, commitMessage="Updated via Apitax"):
        path = self.getPath(path)
        return self.repo.create_file(path, commitMessage, content)

    def updateFile(self, originalFile, newContent, commitMessage="Updated via Apitax"):
        path = self.getPath(originalFile.path)
        return self.repo.update_file(path, commitMessage, newContent, originalFile.sha)

    def deleteFile(self, path=None, file=None, commitMessage="Updated via Apitax"):
        if (not file):
            file = self.isFileExists(path)
        if (not file):
            return False
        path = self.getPath(file.path)
        return self.repo.delete_file(path, commitMessage, file.sha)

    def getFile(self, path):
        return self.isFileExists(path)

    def getFiles(self, path="", recursive=True):
        path = self.getPath(path)
        foundFiles = self.repo.get_contents(path)
        if (recursive):
            files = []
            for file in foundFiles:
                files += self.getFilesHelper(file)
            return files
        else:
            if (isinstance(foundFiles, list)):
                return foundFiles
            return [foundFiles]

    def getFilesHelper(self, file):
        if (file.type == 'file'):
            return [file]
        files = []
        for nextFile in self.repo.get_contents(file.path):
            files += self.getFilesHelper(nextFile)
        return files

    def getFileContent(self, path=None, file=None):
        if (not file):
            file = self.isFileExists(path)
        if (not file):
            return False
        if (self.isEncoded(file)):
            return self.decode(file.content)
        return file.content



import base64


class Git:
    def __init__(self):
        self.git = None
        self.repo = None
        self.user = None

    # This requires a full name: "ShawnClake/Apitax"
    def setRepo(self, name):
        pass

    def getRepos(self):
        pass

    def printRepos(self):
        pass

    def createOrUpdate(self, path, content, commitMessage="Updated via Apitax"):
        file = self.isFileExists(path)
        if (not file):
            return self.createFile(path, content, commitMessage)
        return self.updateFile(file, content, commitMessage)

    def createIfNotExists(self, path, content, commitMessage="Updated via Apitax"):
        if (not self.isFileExists(path)):
            return self.createFile(path, content, commitMessage)
        return False

    def renameFile(self, originalPath, newPath):
        content = self.getFileContent(path=originalPath)
        self.deleteFile(path=originalPath, commitMessage="Updated via Apitax for renaming")
        self.createFile(newPath, content, commitMessage="Updated via Apitax for renaming")
        return True

    def isFileExists(self, path):
        pass

    def createFile(self, path, content, commitMessage="Updated via Apitax"):
        pass

    def updateFile(self, originalFile, newContent, commitMessage="Updated via Apitax"):
        pass

    def deleteFile(self, path=None, file=None, commitMessage="Updated via Apitax"):
        pass

    def getFile(self, path):
        return self.isFileExists(path)

    def getFiles(self, path="", recursive=True):
        pass

    def getFilesHelper(self, file):
        pass

    def getFileContent(self, path=None, file=None):
        pass

    def getPath(self, path):
        if (path[0:1] == '/'):
            return path
        return "/" + path

    def isEncoded(self, file):
        return bool(file.encoding)

    def encode(self, message):
        return base64.b64encode(message.encode())

    def decode(self, message):
        return base64.b64decode(message).decode()  # The last decode turns \n's etc. into actual new lines



from apitax.drivers.Driver import Driver
from apitax.utilities.Files import getAllFiles
from apitax.ah.Options import Options
from pathlib import Path
from apitax.ah.Credentials import Credentials
from apitax.utilities.Json import read
from apitax.ah.State import State
from apitax.utilities.Files import getPath


class OpenstackDriver(Driver):

    def __init__(self):
        super().__init__()
        self.users = read(getPath(State.paths['root'] + "/app/users.json"))

    def getToken(self, response):
        return response.getResponseHeaders().get('X-Subject-Token')

    def getTokenAuthHeader(self, credentials):
        return {'X-Auth-Token': credentials.token}

    def getPasswordAuthData(self, credentials):
        authObj = {'auth': {'identity': {'methods': ['password'], 'password': {
            'user': {'domain': {'id': 'default'}, 'password': credentials.password, 'name': credentials.username}}}}}
        if ("project_id" in credentials.extra):
            authObj['auth'].update({"scope": {"project": {"id": credentials.extra['project_id']}}})
        return authObj

    def isCredentialsPosted(self):
        return True

    def isConfigurable(self):
        return True

    def getScriptsCatalog(self):
        files = getAllFiles(self.config.path + "/grammar/scripts/**/*.ah")
        returner = {"scripts": []}
        for file in files:
            returner['scripts'].append({"label": file.split('/')[-1].split('.')[0].title(), "relative-path": file,
                                        "path": str(Path(file).resolve())})
        # print(returner)
        return returner

    def getCatalog(self, auth):
        from apitax.ah.Connector import Connector
        import json

        connector = Connector(credentials=Credentials(token=auth.token),
                              command="custom --get --driver OpenstackDriver --url " + self.getCatalogEndpoint(),
                              options=Options(debug=False, sensitive=True, driver='OpenstackDriver'), parameters=None)

        commandHandler = connector.execute()

        services = json.loads(commandHandler.getRequest().getResponseBody())

        catalog = {}
        catalog['endpoints'] = {}

        for service in services['catalog']:
            endpoints = service['endpoints']
            if (len(endpoints) > 0):
                for endpoint in endpoints:
                    if (endpoint['interface'] == 'public'):
                        name = service['name']
                        catalog['endpoints'].update({name: {"label": name, "value": endpoint['url']}})

        catalog['selected'] = "http://172.25.190.14:5000"

        return catalog
        
    def isApiAuthenticated(self):
        return True
        
    def piggyBackOffApiAuth(self):
        return True

    def apitaxAuth(self, authObj):
        authObj = authObj['credentials']
        authRequest = ['apiAuthRequest']
        #print(authRequest)
        try:
            return self.users[authObj.username]['role']
        except:
            return None
        return None



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='apitaxdrivers',
    packages=find_packages(),  # this must be the same as the name above
    version='0.0.10',
    description='A set of example and micro drivers which can be used as the basis of more robust API drivers or combined together to form a robust application.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Shawn Clake',
    author_email='shawn.clake@gmail.com',
    url='https://github.com/ShawnClake/Apitax',  # use the URL to the github repo
    keywords=['restful', 'api', 'commandtax', 'scriptax', 'apitax', 'drivers', 'plugins'],  # arbitrary keywords
    include_package_data=True,
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    install_requires=[
        'requests',
        'python-gitlab',
        'pygithub',
        'gitpython',
        'apitaxcore==3.0.6',
        'commanndtax==0.0.8',
        'scriptax==0.0.4',
    ],
)



# Application import
from apitax.ah.commandtax.Request import Request


# Handles Openstack commands related to domains
class Domains(Request):
    def __init__(self, header, debug, sensitive):
        Request.__init__(self, '', header.get(), '', debug=debug, sensitive=sensitive)

    def handle(self, command):
        if (command[0] == 'list'):
            if (len(command) > 1 and command[1] == 'all'):
                self.getAll()
            else:
                self.getMy()
        elif (command[0] == 'create'):
            self.create(command[1])

    def getAll(self):
        print('response: domain list all \n')

    # self.setUrl('http://172.25.190.14:5000/v3/projects')
    # self.setParamData({'is_domain': True})
    # self.get()

    def getMy(self):
        print('response: domain list \n')

    # self.setUrl('http://172.25.190.14:5000/v3/auth/domains')
    # self.get()

    def create(self, name, description="", enabled=True):
        print('response: domain create \n')
    # self.setUrl('http://172.25.190.14:5000/v3/domains')
    # self.setPostData({'domain': { 'name': name, 'description': description, 'enabled':enabled}})
    # self.post()
